// Class: Example3
//
// Author: Alyce Brady
//
// License Information:
//   This class is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation.
//
//   This class is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

import javax.swing.JMenu;

import edu.kzoo.grid.ColorBlock;
import edu.kzoo.grid.Grid;
import edu.kzoo.grid.Location;
import edu.kzoo.grid.display.ColorBlockDisplay;
import edu.kzoo.grid.display.DisplayMap;

import edu.kzoo.grid.gui.ControlButton;
import edu.kzoo.grid.gui.EnabledDisabledStates;
import edu.kzoo.grid.gui.GridAppFrame;
import edu.kzoo.grid.gui.ThreadedControlButton;

import edu.kzoo.grid.gui.nuggets.BGColorChoiceMenu;
import edu.kzoo.grid.gui.nuggets.BasicHelpMenu;
import edu.kzoo.grid.gui.nuggets.ClearGridButton;
import edu.kzoo.grid.gui.nuggets.ColorChoiceMenu;
import edu.kzoo.grid.gui.nuggets.MinimalFileMenu;
import edu.kzoo.grid.gui.nuggets.NewBoundedGridButton;

/**
 *  Grid Package Examples:<br>
 * 
 *    The Example3 class constructs a graphical user interface that
 *    includes a New Grid button, a background color chooser, a
 *    Reset button, and a group of buttons generated automatically
 *    by looking at the methods of another class.
 *
 *  @author Alyce Brady
 *  @version 15 August 2004
 **/
public class Example3
{
    // Specify attributes of the grid display.
    private static final int MAX_WIDTH = 400;
    private static final int MAX_HEIGHT = 400;
    private static final int MIN_CELL_SIZE = 20;
    private static final boolean DISPLAY_GRID_AFTER_BUTTON_PRESSES = true;

    /**
     *  Start the Example3 program.
     *  The String arguments (args) are not used in this application.
     **/
    public static void main(String[] args)
    {
        // Construct a window in which to display a grid with color blocks.  
        GridAppFrame gui = new GridAppFrame();
        DisplayMap.associate("edu.kzoo.grid.ColorBlock",
                             new ColorBlockDisplay());

        // Include a minimal file menu with a Quit option.
        gui.includeMenu(new MinimalFileMenu());

        // Include a help menu with information about the application.
        JMenu helpMenu = new BasicHelpMenu(
            "Example 3: Using Control Buttons and Other Components " +
            "within the Grid Package",
            "Alyce Brady",
            "The Grid Package was inspired by the AP Marine Biology " +
            "Simulation Case Study,<br> whose graphical user interface " +
            "code was written primarily by Julie Zelenski.", "15 August 2004");
        gui.includeMenu(helpMenu);

        // Include a New Grid button.
        gui.includeControlComponent(new NewBoundedGridButton(gui, "New Grid"),
                                    EnabledDisabledStates.NEEDS_APP_WAITING);

        // Include a background color chooser component.
        ColorChoiceMenu bgColorChooser = new BGColorChoiceMenu(gui);
        gui.includeControlComponent(bgColorChooser,
                                    EnabledDisabledStates.NEEDS_APP_WAITING);

        // Include a color chooser component for color blocks.
        ColorChoiceMenu blockColorChooser = new ColorChoiceMenu("Fill Color");
        gui.includeControlComponent(blockColorChooser,
                                    EnabledDisabledStates.NEEDS_APP_WAITING);

        // Include a button that fills the grid with color blocks.
        gui.includeControlComponent(new FillButton(gui, blockColorChooser),
                            EnabledDisabledStates.NEEDS_GRID_AND_APP_WAITING);

        // Include a button that empties the grid.
        gui.includeControlComponent(
                        new ClearGridButton(gui, "Empty Grid",
                                            DISPLAY_GRID_AFTER_BUTTON_PRESSES),
                        EnabledDisabledStates.NEEDS_GRID_AND_APP_WAITING);

        // Include a button that empties the grid and resets the speed slider.
        gui.includeControlComponent(new ResetButton(gui),
                            EnabledDisabledStates.NEEDS_GRID_AND_APP_WAITING);

        // Include a speed slider to control the speed of the application.
        gui.includeSpeedSlider();

        // Construct the visible window with the specified components.
        gui.constructWindowContents("Example 3: Using Simple Components",
                                    bgColorChooser.currentColor(),
                                    MAX_WIDTH, MAX_HEIGHT, MIN_CELL_SIZE);
    }


  // Nested classes for Fill and Reset buttons

    /** A FillButton object fills the grid with color blocks.
     **/
    private static class FillButton extends ThreadedControlButton
    {
        // Don't need to redisplay after fill because the act method displays
        // the grid as it goes.
        private static final boolean DISPLAY_GRID_AFTER_FILL = false;

        // Instance variable
        private ColorChoiceMenu colorChooser;

        /** Constructs a button that fills a grid with color blocks.
         *    @param gui    graphical user interface containing this button
         *    @param colorChooser menu specifying color to use for color blocks
         **/
        private FillButton(GridAppFrame gui, ColorChoiceMenu colorChooser)
        {
            super(gui, "Fill Grid", DISPLAY_GRID_AFTER_FILL);
            this.colorChooser = colorChooser;
        }

        /** Fills the grid with color blocks, displaying the grid after
         *  each block is placed.
         **/
        public void act()
        {
            Grid grid = getGUI().getGrid();
            for ( int row = 0; row < grid.numRows(); row++ )
            {
                for ( int col = 0; col < grid.numCols(); col++ )
                {
                    Location loc = new Location(row, col);
                    grid.remove(loc);
                    grid.add(new ColorBlock(colorChooser.currentColor()), loc);
                    getGUI().showGrid();
                }
            }
        }
    } 

    /** A ResetButton object sets the grid to null and the speed slider bar
     *  to its initial value.
     **/
    private static class ResetButton extends ControlButton
    {
        /** Constructs a button that removes the grid and resets the
         *  speed slider bar.
         *    @param gui    graphical user interface containing this button
         **/
        private ResetButton(GridAppFrame gui)
        {
            super(gui, "Reset", DISPLAY_GRID_AFTER_BUTTON_PRESSES);
        }

        /** Sets the grid to null and the speed slider bar to its initial value.
         **/
        public void act()
        {
            getGUI().setGrid(null);
            getGUI().resetDelay();
        }
    } 

}
